@@ -9,6 +9,7 @@ IMPORTANT:
 import math
 
 from photonvision import PhotonUtils
+from photonvision._photonvision import RobotPoseEstimator, PoseStrategy
 
 from robotpy_toolkit_7407.sensors.photonvision.photon_target import PhotonTarget, AprilTag
 from robotpy_toolkit_7407.sensors.photonvision.photon_camera import PhotonCamera
@@ -66,6 +67,20 @@ class PhotonOdometry:
             fieldToTarget=field_to_target,
         )
 
+        estimated_field_to_robot_pose = RobotPoseEstimator(
+            self.field_layout,
+            PoseStrategy.CLOSEST_TO_REFERENCE_POSE,
+            [
+                (
+                    self.camera,
+                    Transform3d(
+                        self.camera.camera_to_robot_pose.translation(),
+                        self.camera.camera_to_robot_pose.rotation()
+                    )
+                )
+            ]
+        )
+
         print("ESTIMATED FIELD TO ROBOT POSE: ", estimated_field_to_robot_pose)
 
         gyro_angle = self.gyro.get_robot_heading()
@@ -78,35 +93,39 @@ class PhotonOdometry:
         field_to_camera = (
             field_to_target.x - camera_to_target.x,
             field_to_target.y - camera_to_target.y
+
         )
 
-        field_to_robot = (
-            field_to_camera[0] - h * math.sin(theta),
-            field_to_camera[1] - h * math.cos(theta)  # Testing positive did not work
-        )
+    field_to_robot = (
+        field_to_camera[0] - h * math.sin(theta),
+        field_to_camera[1] - h * math.cos(theta)  # Testing positive did not work
 
-        return field_to_robot
+    )
 
-    def parse_field_layout(self, field_layout: dict):
-        new_layout: AprilTagFieldLayout = None
-        april_tags: list[AprilTag] = []
-        length: float = None  # meters
-        width: float = None  # meters
 
-        for i in field_layout['apriltags'].keys():
-            april_tags.append(
-                AprilTag(
-                    ID=i,
-                    pose=field_layout['apriltags'][i]
-                )
+return field_to_robot
+
+
+def parse_field_layout(self, field_layout: dict):
+    new_layout: AprilTagFieldLayout = None
+    april_tags: list[AprilTag] = []
+    length: float = None  # meters
+    width: float = None  # meters
+
+    for i in field_layout['apriltags'].keys():
+        april_tags.append(
+            AprilTag(
+                ID=i,
+                pose=field_layout['apriltags'][i]
             )
-
-        length = field_layout['fieldLength']
-        width = field_layout['fieldWidth']
-        new_layout = AprilTagFieldLayout(
-            apriltags=april_tags,
-            fieldLength=length,
-            fieldWidth=width
         )
 
-        return new_layout
+    length = field_layout['fieldLength']
+    width = field_layout['fieldWidth']
+    new_layout = AprilTagFieldLayout(
+        apriltags=april_tags,
+        fieldLength=length,
+        fieldWidth=width
+    )
+
+    return new_layout
